#
# This is file for drbl vote zone generating
# $Id: do_vote.pl,v 2.10 2003/10/27 14:48:14 agk Exp $
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   Copyright 2002, Koulik Andrei <agk@agk.nnov.ru>
#

use strict;
use POSIX;

# Code parameters

my $defalutConfig = '/usr/local/etc/drblnode.conf';

# global declarations
my %Config;
my %masterIP;
keys(%masterIP) = 256;

my %zoneContent;

my ($Id,$Revision) = ('$Id: do_vote.pl,v 2.10 2003/10/27 14:48:14 agk Exp $','$Revision: 2.10 $');

require '/usr/local/share/drbl/config.pl';
require '/usr/local/share/drbl/drbllib.pl';

cfgDoFile( (defined($ARGV[0])) ? $ARGV[0] : $defalutConfig, \%Config )||
  die "cannot parse the config file\n";

my $maxNets = $Config{'aggr_net_number'};

$maxNets = 256 unless $maxNets;

#Build  Vote zone

# walk through all vote zone
while( my($zn,$hp) = each( %{$Config{'vote'}} ) ) {
  my $voteFile = $hp->{'output'};
  my $header = cfgGetAltValue('header',$hp,$Config{'defaults'});

  next if defined $hp->{'skip'};
  %masterIP = ();
  outNot("Process zone: $zn to file $voteFile\n");
  if ( not open(OVOTE,">$voteFile") ) {
    outErr("Can't open vote file $voteFile: $!");
    die "Fatal error\n";
  }
  outHeader(\*OVOTE,$header) if $header;
  outInfoHeader(\*OVOTE);
  # drain each source
  foreach my $src (sort keys %{$hp->{'src'}}) {
    my($res,$zone,$master,$dig,$target);

    outDeb("source: $src");
    $res = parseTarget($hp->{'src'}->{$src},\$zone,\$master);
    if (not $res) {
      outErr("Bad source $src of $zn: ".$hp->{'src'}->{$src});
      die "Fatal error\n";
    }
    if ( $res != 5 ) {
      $dig = $Config{'dig'}." \@$master $zone axfr|";
      $target = "$zone\@$master";
    } else {
      $dig = "<$zone";
      $target = $zone;
    }
    outNot("Loading ($target).....");
    if ( defined $zoneContent{$target} ) {
      outNot("Cached");
    } else {
      $zoneContent{$target} = suck_zone2($dig);
    }
    outNot("Ok");
    if( not $zoneContent{$target} ) {
      outErr("Can't read from $dig");
      die "Fatal error\n";
    }
    while( my($addr,$ac) = each ( %{$zoneContent{$target}})){
      unless ($addr =~ /^[0-9]+(\.[0-9]+){0,3}$/) {
	outNot("Rejected: Bad IP: $addr") unless $addr eq '@';
	next;
      }
      putNetwork(\%masterIP, $ac->{'A'} ,$ac->{'TXT'}, split('\.',$addr) );
    }
  }
  outDeb("Dump zone");
  dumpNetLayer(\*OVOTE,\%masterIP);
  close(OVOTE);
}

sub outInfoHeader {
  my ($ostream) = @_;
  print $ostream "\tIN\tTXT\t\"DRBL-Version=$Revision\"\n";
  print $ostream "\tIN\tTXT\t\"DRBL-Type=VOTE\"\n";
  print $ostream "\tIN\tTXT\t\"DRBL-Info=" . &cfgGetValue(\%Config,'info.target')."\"\n";
  my $Date = ctime(time);
  chomp($Date);
  print $ostream "\tIN\tTXT\t\"DRBL-Date=$Date\"\n";
}

sub putNetwork {
  my($hp,$right,$txt,@net) = @_;

#  outDeb("putNet params: [".join('][',@_)."]");
  my $netsq = pop @net;
  if ( not defined $hp -> {$netsq} ) {
    $hp -> {$netsq} = {
		       'netcount' => 0,
		       'dump' => 'NO',
		       'nets' => {},
		       'txt' => {},
		       'right' => '127.0.0.2'
		       }
  }
  my $cn = $hp -> {$netsq};
  return if $cn -> {'dump'} eq 'OK';
  if ( $#net >= 0 ) {
    $cn -> {'netcount'}++ if not defined $cn -> {'nets'} -> {$net[$#net]};
    if ( $cn -> {'netcount'} >= $maxNets ) {
      $cn -> {'dump'} = 'OK';
      $cn -> {'txt'} -> {'Aggregating network'} = '';
      outNot("Aggregating network added");
    } else {
      putNetwork($cn -> {'nets'},$right,$txt,@net);
    }
  } else {
    $cn -> {'dump'} = 'OK';
    $cn -> {'right'} = $right;
    if (defined $txt ) {
      foreach my $textitem (keys %$txt) {
	$cn -> {'txt'} -> {$textitem} = '';
      }
    }
  }
}

sub dumpNetLayer {
  my($ostr,$hp,$sufx) = @_;

  my $pref = (($sufx =~ tr/\./\./) == 3) ? '':'*.';
  while( my($netsq,$lp) = each(%$hp)) {
 #   outDeb("check: $netsq");
    if ( $lp -> {'dump'} eq 'OK' ) {
      print $ostr "$pref$netsq$sufx\tIN\tA\t".$lp->{'right'}."\n";
      foreach (%{$lp->{'txt'}})  { print $ostr "\tIN\tTXT\t\"$_\"\n" unless $_ eq ""; };
    } else {
      dumpNetLayer($ostr,$lp->{'nets'},'.' . $netsq . $sufx);
    }
  }
}
